@@ -21,11 +21,19 @@
 #include "eog-pixbuf-util.h"
 #include "eog-metadata-reader.h"
 #include "eog-image-save-info.h"
+#include "eog-info-view-file.h"
 #include "eog-util.h"
 #if HAVE_JPEG
 #include "eog-image-jpeg.h"
 #endif
 
+#if HAVE_LCMS
+ #include <lcms.h>
+ #ifndef EXIF_TAG_GAMMA
+  #define EXIF_TAG_GAMMA 0xa500
+ #endif
+#endif
+
 enum {
 	SIGNAL_LOADING_UPDATE,
 	SIGNAL_LOADING_SIZE_PREPARED,
@@ -77,6 +85,12 @@ eog_image_dispose (GObject *object)
 
 	eog_image_free_mem_private (EOG_IMAGE (object));
 
+	
+	if (priv->toclean) {
+		g_free (priv->toclean);
+		gnome_vfs_monitor_cancel (priv->monitor_handle);
+	}
+	
 	if (priv->uri) {
 		gnome_vfs_uri_unref (priv->uri);
 		priv->uri = NULL;
@@ -251,6 +265,9 @@ eog_image_instance_init (EogImage *img)
 	priv->load_finished = NULL;
 #if HAVE_EXIF
 	priv->exif = NULL;
+#endif
+#if HAVE_LCMS
+	priv->profile = NULL;
 #endif
 	priv->data_ref_count = 0;
 
@@ -549,6 +566,142 @@ eog_image_load_exif_data_only (EogImage *img, EogJob *job, GError **error)
 	return !failed;
 }
 
+#if HAVE_LCMS
+void
+eog_image_apply_display_profile (EogImage *img, cmsHPROFILE screen)
+{
+	EogImagePrivate *priv;
+	cmsHTRANSFORM transform;
+	int row, width, rows, stride;
+	guchar *p;
+	
+	g_return_if_fail (img != NULL);
+
+	if (screen == NULL)
+		return;
+
+	priv = img->priv;
+	if (priv->profile == NULL)
+		return;
+	
+	transform = cmsCreateTransform(priv->profile, TYPE_RGB_8, screen, TYPE_RGB_8, INTENT_PERCEPTUAL, 0);
+	
+	rows = gdk_pixbuf_get_height(priv->image);
+	width = gdk_pixbuf_get_width (priv->image);
+	stride = gdk_pixbuf_get_rowstride (priv->image);
+	p = gdk_pixbuf_get_pixels (priv->image);
+	for (row = 0; row < rows; ++row) {
+		cmsDoTransform(transform, p, p, width);
+		p += stride;
+	}
+	cmsDeleteTransform(transform);
+	//cmsCloseProfile(screen);
+}
+
+static void
+extract_profile (EogImage *img, EogMetadataReader *md_reader)
+{
+	EogImagePrivate *priv = img->priv;
+#if HAVE_EXIF
+	ExifEntry *entry;
+	const ExifByteOrder o = exif_data_get_byte_order (priv->exif);
+#endif
+	/* TODO: switch on format to specialised functions */
+
+	/* Embedded ICC profiles rule over anything else */
+	{
+		gpointer data = eog_metadata_reader_get_icc_chunk (md_reader);
+		if (data != NULL) {
+			cmsErrorAction (LCMS_ERROR_SHOW);
+			priv->profile = cmsOpenProfileFromMem(data, eog_metadata_reader_get_icc_chunk_size (md_reader));
+			if (priv->profile) {
+				g_printerr("JPEG has ICC profile\n");
+			} else {
+				g_printerr("JPEG has invalid ICC profile\n");
+			}
+			return;
+		}
+	}
+#if HAVE_EXIF
+	/* No EXIF data, so can't do anything */
+	if (priv->exif == NULL)
+		return;
+	
+	entry = exif_content_get_entry (priv->exif->ifd [EXIF_IFD_EXIF], EXIF_TAG_COLOR_SPACE);
+	if (entry == NULL)
+		return;
+
+	if (exif_get_short (entry->data, o) == 1) {
+		priv->profile = cmsCreate_sRGBProfile ();
+		g_printerr ("JPEG is sRGB\n");
+	} else if (exif_get_short (entry->data, o) == 2) {
+		/* TODO: create Adobe RGB profile */
+		//priv->profile = cmsCreate_Adobe1998Profile ();
+		g_printerr ("JPEG is Adobe RGB (NOT correcting for now!)\n");
+	} else if (exif_get_short (entry->data, o) == 0xFFFF) {
+		double gammaValue;
+		cmsCIExyY whitepoint;
+		cmsCIExyYTRIPLE primaries;
+		LPGAMMATABLE gamma[3];
+		
+		const int offset = exif_format_get_size (EXIF_FORMAT_RATIONAL);
+		ExifRational r;
+		
+		entry = exif_content_get_entry (priv->exif->ifd [EXIF_IFD_0], EXIF_TAG_WHITE_POINT);
+		if (entry && entry->components == 2) {
+			r = exif_get_rational (entry->data, o);
+			whitepoint.x = (double)r.numerator/r.denominator;
+			r = exif_get_rational (entry->data + offset, o);
+			whitepoint.y = (double)r.numerator/r.denominator;
+			whitepoint.Y = 1.0;
+		} else {
+			g_printerr("No whitepoint found\n");
+			return;
+		}
+		    
+		entry = exif_content_get_entry (priv->exif->ifd [EXIF_IFD_0], EXIF_TAG_PRIMARY_CHROMATICITIES);
+		if (entry && entry->components == 6) {
+			r = exif_get_rational (entry->data + 0 * offset, o);
+			primaries.Red.x = (double)r.numerator/r.denominator;
+			r = exif_get_rational (entry->data + 1 * offset, o);
+			primaries.Red.y = (double)r.numerator/r.denominator;
+		      
+			r = exif_get_rational (entry->data + 2 * offset, o);
+			primaries.Green.x = (double)r.numerator/r.denominator;
+			r = exif_get_rational (entry->data + 3 * offset, o);
+			primaries.Green.y = (double)r.numerator/r.denominator;
+		      
+			r = exif_get_rational (entry->data + 4 * offset, o);
+			primaries.Blue.x = (double)r.numerator/r.denominator;
+			r = exif_get_rational (entry->data + 5 * offset, o);
+			primaries.Blue.y = (double)r.numerator/r.denominator;		    
+		      
+			primaries.Red.Y = primaries.Green.Y = primaries.Blue.Y = 1.0;
+		} else {
+			g_printerr("No primary chromaticities found\n");
+			return;
+		}
+
+		entry = exif_content_get_entry (priv->exif->ifd [EXIF_IFD_EXIF], EXIF_TAG_GAMMA);
+		if (entry) {
+			r = exif_get_rational (entry->data, o);
+			gammaValue = (double)r.numerator/r.denominator;
+		} else {
+			/* Assume 2.2 */
+			g_printerr("No gamma found\n");
+			gammaValue = 2.2;
+		}
+		    
+		gamma[0] = gamma[1] = gamma[2] = cmsBuildGamma(256, gammaValue);
+		    
+		priv->profile = cmsCreateRGBProfile(&whitepoint, &primaries, gamma);
+		g_printerr ("JPEG is calibrated\n");
+		cmsFreeGamma(gamma[0]);
+	}
+#endif
+}
+#endif
+
 /* this function runs in it's own thread */
 static gboolean
 eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error)
@@ -679,8 +832,12 @@ eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error
 		if (format != NULL) {
 			priv->file_type = gdk_pixbuf_format_get_name (format);
 		}
+#if HAVE_LCMS
+		if (md_reader != NULL) {
+			extract_profile (img, md_reader);
+		}
+#endif
 	}
-	
 	/* clean up */
 	g_object_unref (loader);
 	if (md_reader != NULL) {
@@ -844,6 +1001,16 @@ eog_image_get_pixbuf (EogImage *img)
 	return image;
 }
 
+#if HAVE_LCMS
+cmsHPROFILE
+eog_image_get_profile (EogImage *img)
+{
+	g_return_val_if_fail (EOG_IS_IMAGE (img), NULL);
+
+	return img->priv->profile;
+}
+#endif
+
 GdkPixbuf* 
 eog_image_get_pixbuf_thumbnail (EogImage *img)
 {
@@ -1465,6 +1632,12 @@ eog_image_free_mem_private (EogImage *image)
 		}
 		priv->exif_chunk_len = 0;
 
+#if HAVE_LCMS
+		if (priv->profile != NULL) {
+		  cmsCloseProfile (priv->profile);
+		}
+#endif
+		
 		priv->status = EOG_IMAGE_STATUS_UNKNOWN;
 	}
 }
@@ -1568,6 +1741,115 @@ eog_image_get_uri (EogImage *img)
 	return gnome_vfs_uri_ref (img->priv->uri);
 }
 
+typedef struct {
+	EogInfoViewFile *view;
+	EogImage *img;
+} VfsFileCbContext;
+
+
+static gchar* temp = NULL;
+static gboolean had_create = FALSE, had_delete = FALSE;
+
+static void
+vfs_monitor_dir_cb (GnomeVFSMonitorHandle *handle,
+          const gchar *monitor_uri,
+          const gchar *info_uri,
+          GnomeVFSMonitorEventType event_type,
+          gpointer user_data)
+{
+	VfsFileCbContext *ctx = user_data;
+	gchar *monitor_ptr = monitor_uri;
+	gchar *info_ptr = info_uri;
+	
+	monitor_ptr += 7;
+	info_ptr += 7;
+		
+	g_return_if_fail (EOG_IS_IMAGE (ctx->img));
+	g_return_if_fail (EOG_IS_INFO_VIEW_FILE (ctx->view));
+	
+	if (strcmp (gnome_vfs_uri_extract_dirname (ctx->img->priv->uri), monitor_ptr) == 0 &&
+		strcmp (gnome_vfs_uri_get_path (ctx->img->priv->uri), info_ptr) == 0 &&
+		event_type == GNOME_VFS_MONITOR_EVENT_DELETED) {
+		
+		had_delete = TRUE;
+	}
+	
+	if (strcmp (gnome_vfs_uri_extract_dirname (ctx->img->priv->uri), monitor_ptr) == 0 &&
+		event_type == GNOME_VFS_MONITOR_EVENT_CREATED) {
+		
+		if (temp)
+			g_free (temp);
+		had_create = TRUE;
+		
+		temp = g_strdup (info_uri);
+	}
+}
+
+static void
+vfs_monitor_file_cb (GnomeVFSMonitorHandle *handle,
+          const gchar *monitor_uri,
+          const gchar *info_uri,
+          GnomeVFSMonitorEventType event_type,
+          gpointer user_data)
+{
+	VfsFileCbContext *ctx = user_data;
+
+	g_return_if_fail (EOG_IS_IMAGE (ctx->img));
+	g_return_if_fail (EOG_IS_INFO_VIEW_FILE (ctx->view));
+	
+	if (had_delete && had_create)
+	{
+		if (ctx->img->priv->caption)
+			g_free (ctx->img->priv->caption);
+		ctx->img->priv->caption = NULL;
+		gnome_vfs_uri_unref (ctx->img->priv->uri);
+		ctx->img->priv->uri = gnome_vfs_uri_new (temp);
+		eog_info_view_file_show_data (ctx->view, ctx->img);
+
+		/* g_print ("Rename detected to %s -- %s\n", 
+			temp, eog_image_get_caption (ctx->img)); */
+		
+		gnome_vfs_monitor_add  (&ctx->img->priv->monitor_handle, 
+			gnome_vfs_uri_get_path (ctx->img->priv->uri),
+			GNOME_VFS_MONITOR_FILE,
+			vfs_monitor_file_cb,
+			ctx);
+	}
+	
+	had_create = FALSE;
+	had_delete = FALSE;
+	
+	return;
+}
+
+void 
+eog_image_register_infoview (EogImage *img, gpointer view)
+{
+	VfsFileCbContext *ctx = g_new0 (VfsFileCbContext, 1);
+	
+	ctx->img = img;
+	ctx->view = view;
+
+	g_return_if_fail (EOG_IS_INFO_VIEW_FILE (ctx->view));
+	g_return_if_fail (EOG_IS_IMAGE (ctx->img));
+		
+	gnome_vfs_monitor_add  (&ctx->img->priv->monitor_handle, 
+		gnome_vfs_uri_extract_dirname (ctx->img->priv->uri),
+		GNOME_VFS_MONITOR_DIRECTORY,
+		vfs_monitor_dir_cb,
+		ctx);
+	
+	gnome_vfs_monitor_add  (&ctx->img->priv->monitor_handle, 
+		gnome_vfs_uri_get_path (ctx->img->priv->uri),
+		GNOME_VFS_MONITOR_FILE,
+		vfs_monitor_file_cb,
+		ctx);
+		
+	img->priv->toclean = ctx;	
+}
+
+
+
 gboolean
 eog_image_is_modified (EogImage *img)
 {
@@ -1631,7 +1913,7 @@ EogImage*
 eog_image_data_unref (EogImage *img)
 {
 	g_return_val_if_fail (EOG_IS_IMAGE (img), NULL);
-
+		
 	if (img->priv->data_ref_count > 0) {
 		img->priv->data_ref_count--;
 	}
@@ -1649,3 +1931,57 @@ eog_image_data_unref (EogImage *img)
 	
 	return img;
 }
+
+/* Print API */
+
+void
+eog_image_print (EogImage *img, GnomePrintContext *context, gdouble paper_width, gdouble paper_height)
+{	
+	EogImagePrivate *priv = NULL;
+	GdkPixbuf *printed_image = NULL;
+	gint pix_width;
+	gint pix_height;
+	gdouble width, height;
+
+	g_return_if_fail (EOG_IS_IMAGE (img));
+	
+	priv = img->priv;
+
+	g_return_if_fail (GDK_IS_PIXBUF (priv->image));
+	
+	if (gdk_pixbuf_get_width (priv->image) > gdk_pixbuf_get_height (priv->image)) {
+		printed_image = gdk_pixbuf_rotate_simple (priv->image, 
+			GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE);
+	} else {
+		gdk_pixbuf_ref (priv->image);
+		printed_image = priv->image;
+	}
+
+	pix_width = gdk_pixbuf_get_width (printed_image);
+	pix_height = gdk_pixbuf_get_height (printed_image);
+
+	width = (gint)paper_width;
+	height = (gint)paper_height;
+
+	if (((gdouble) pix_height/pix_width) >
+	    ((gdouble)width/height)) {
+		/* We scale to the top */
+		width = height * (gdouble)pix_width/pix_height;
+	} else {
+		/* We scale to the sides of the page */
+		height = width * (gdouble)pix_height/pix_width;
+	}
+
+	width -= 40;
+	height -= 40;
+	
+	gnome_print_translate (context, (paper_width - width)/2.0, (paper_height - height)/2.0);
+	gnome_print_scale (context, width, height);
+	
+	gnome_print_rgbimage (context, gdk_pixbuf_get_pixels (printed_image), 
+		pix_width, pix_height, gdk_pixbuf_get_rowstride (printed_image));
+		
+  
+	g_object_unref (G_OBJECT (printed_image));	
+}
+
